@@ -5,96 +5,155 @@
 # Date: 2002-Apr-12 20:06 (EDT)
 # Function: Tell someone what happened
 #
-# $Id: Notify.pm,v 1.33 2003/03/30 15:12:31 jaw Exp $
+# $Id: Notify.pm,v 1.76 2008/10/25 16:34:53 jaw Exp $
 
 package Notify;
 use NotMe;
-use Encode;
-use POSIX ('_exit');
+use Argus::Encode;
+use POSIX ('_exit', 'strftime', 'tzset');
 
+use strict;
+
+my $lastupd;		# time of last notification or ack
 my %byid     = (); 
 my %lastsent = ();	# by dest
 my %queue    = ();	# by dest => {not, tag}
 my %unacked  = ();	# by id
+my %dstparam = ();	# probably a bad idea...
+# RSN - persist dstparams in file
 
 # only send so often, otherwise queue
 # can be overridden in user-defined notification methods via 'qtime: 1234'
 my $QUEUETIME = 120;
 
-# we keep re-sending, but if it comes up, give up after this long
-my $UNACK_TO  = 1800;
-
 # if older than this, and no longer active, throw them away
-my $OLD_AGE = 3600 * 24 * 5;
+my $OLD_AGE = 3600 * 24 * 10;
 
 # delay escalation, if no longer down for this long
 my $DELAY_ESCALATE = 600;
 
 # fields which get saved in save file (and then re-loaded)
-my @SAVE_FIELDS =qw(created msg sentcnt state objstate priority autoack
-		    ackonup period ackedby ackedat escalated lastsent);
+my @SAVE_FIELDS = qw(created msg sentcnt state objstate objovstate
+		     priority autoack ackonup period ackedby ackedat
+		     escalated lastsent audit detail severity reason
+		     ackonbetter ackonworse);
 
 sub new {
-    my $obj = shift;
-    my( $me, $msg, $aa, $dst, @t );
-
-    return unless $obj->{notify}{sendnotify};
-
-    # skip if in override
-    return if $me->{obj}->{ovstatus} eq 'override';
+    my $obj   = shift;
+    my %param = @_;
+    my( $me, $msg, $fmt, $aa, $dst, @t );
 
+    # do not notify if notifies are not wanted
+    return unless value_at_severity('sendnotify', $obj);
+    
     if( $obj->{status} eq 'down' ){
 	$msg = $obj->{notify}{messagedn};
-	$aa  = $obj->{notify}{autoack};
+	$fmt = $obj->{notify}{message_fmtdn};
+	$aa  = value_at_severity('autoack', $obj);
     }else{
 	$msg = $obj->{notify}{messageup};
+	$fmt = $obj->{notify}{message_fmtup};
 	$aa  = 1;	# up messages never need to be acked
     }
 
+    $fmt ||= $obj->{notify}{message_fmt};
+    
+    if( $param{audit} ){
+	# notify is an audit channel
+	$msg = 'audit: %A';
+	$aa = 1;
+    }
+    
     # do not notify if user specified an empty msg
     return unless $msg;
-    $msg = expand($msg, $obj);
     
     $me = {
-	obj      => $obj,
-	msg      => $msg,
-	created  => $^T,
-	idno     => newid(),
-	sentcnt  => 0,
-	escalated=> 0,
-	state    => 'active',		# active, supressed, acked
-	objstate => $obj->{status},
-	priority => $obj->{notify}{priority},
-	autoack  => $aa,
-	period   => $obj->{notify}{renotify},
-	ackonup  => $obj->{notify}{ackonup},
-	status   => {},
-	sendto   => [],
-	log      => [],
-
+	obj         => $obj,
+	created     => $^T,
+	idno        => newid(),
+	sentcnt     => 0,
+	escalated   => 0,
+	state       => 'active',		# active, supressed, acked
+	objstate    => $obj->{status},
+	objovstate  => $obj->{ovstatus},
+	priority    => value_at_severity('priority', $obj),
+	autoack     => $aa,
+	period      => value_at_severity('renotify', $obj),
+	ackonup     => value_at_severity('ackonup',  $obj),
+	ackonbetter => value_at_severity('ack_on_better', $obj),
+	ackonworse  => value_at_severity('ack_on_worse',  $obj),
+	status      => {},
+	sendto      => [],
+	log         => [],
+	audit       => $param{audit},	# bool
+	detail      => $param{detail},
+	severity    => $obj->{currseverity},
+	reason      => $obj->{srvc}{reason},
     };
-    bless $me;
+    bless $me; # gesundheit
+
+    $me->{msg} = $me->expand($fmt, $msg, $obj);
+    
+    if( $me->can('notify_policy') ){
+	return unless $me->notify_policy();
+    }
+    
     return unless $me->init($obj);
     
-    # RSN - supress everything else regarding this object
-     
     $me->loggit( 'system', 'created' );
     $me->notify();
 }
 
+# return notify::field.severity // notify::field
+sub value_at_severity {
+    my $fld = shift;
+    my $obj = shift;
+    my $sev = shift;
+
+    $sev ||= $obj->{currseverity};
+
+    my $v = $obj->{notify}{"$fld.$sev"};
+    return $v if defined $v;
+    return $obj->{notify}{$fld}
+}
+
 sub init {
     my $me  = shift;
     my $obj = shift;
     my( $notify );
 
-    # use notifyup, notifydn, or notify?
-    $notify = $obj->{notify}{$me->{objstate} eq 'down' ? 'notifydn' : 'notifyup'};
-    $notify = $obj->{notify}{notify} unless defined $notify;
+    # use notifyup, notifydn, (...), or notify?
+    if( $me->{audit} ){
+	$notify = $obj->{notify}{notifyaudit};
+    }else{
+	# notify.severity // notify(up|dn) // notify
+	if( $me->{objstate} eq 'down' ){
+	    $notify = $obj->{notify}{"notify.$me->{severity}"};
+	    $notify = $obj->{notify}{notifydn} unless defined $notify;
+	}else{
+	    $notify = $obj->{notify}{"notify.clear"};
+	    $notify = $obj->{notify}{notifyup} unless defined $notify;
+	}
+
+	$notify = $obj->{notify}{notify} unless defined $notify;
+	$notify .= ' ' . $obj->{notify}{notifyalso} if $obj->{notify}{notifyalso};
+    }
+
+    {
+	my $n;
+	foreach my $dst ( split /\s+/, $notify ){
+	    # QQQ - is this really a good idea?
+	    $dst = $dstparam{$dst}{redirect} if $dstparam{$dst} && $dstparam{$dst}{redirect};
+	    $n .=  ($n ? ' ' : '') . $dst;
+	}
+	$notify = $n;
+    }
     
     # set status of each rcpt
     $me->{sendto} = [ { n => 0, start => 0, who => [] } ];
     foreach my $dst ( split /\s+/, $notify ){
 	next if $dst eq 'none';
+	next unless $dst;
 	$me->{status}{$dst} ||= 'created';
 	push @{$me->{sendto}[0]{who}}, $dst;
     }
@@ -111,26 +170,36 @@ sub init {
     # pre-build escalation table
     # A wicked messenger falleth into mischief: but a faithful ambassador is health.
     #   -- proverbs 13:17
-    my $n = 1;
-    if( $obj->{notify}{escalate} ){
-	foreach my $esc ( split /\;\s+/, $obj->{notify}{escalate} ){
-	    # each is of form N dest dest dest ...
-	    # previously, N was number of pages to escalate
-	    # currently, it is elapsed-time in minutes
-	    # QQQ - is there a preference?
-	    my @a = split /\s+/, $esc;
-	    next unless @a;
-	    my $nt = shift @a;
-	    $nt *= 60;  # minutes -> seconds
-	    next unless $nt;
-	    
-	    $me->{sendto}[$n] = { n => $n, start => $nt, who => [@a] };
-	    $n ++;
+    unless( $me->{autoack} ){
+	my $n = 1;
+	my $esc = value_at_severity('escalate', $obj, $me->{severity});
+	if( $esc ){
+	    foreach my $esc ( split /\;\s+/, $esc ){
+		# each is of form N dest dest dest ...
+		# previously, N was number of pages to escalate
+		# then, it was elapsed-time in minutes
+		# currently, a timespec, defaulting to minutes if no units are specified.
+		# QQQ - is there a preference?
+		my @a = split /\s+/, $esc;
+		next unless @a;
+		my $nt = shift @a;
+		eval { $nt = ::timespec($nt, 60); };
+		if($@){
+		    ::loggit( "invalid timespec ($nt) for escalation.", 1 );
+		}
+		next unless $nt;
+		
+		$me->{sendto}[$n] = { n => $n, start => $nt, who => [@a] };
+		$n ++;
+	    }
 	}
     }
 
-    push @{$obj->{notify}{list}}, $me;
+    push @{$obj->{notify}{list}}, $me
+	unless $me->{audit};		# do not list audit msgs in object notify list
+    
     $byid{$me->{idno}} = $me;
+    $lastupd = $^T;
     
     if( $me->{state} eq 'active' ){
 	$unacked{$me->{idno}} = $me;
@@ -141,6 +210,7 @@ sub init {
     $me->{acl_ntfydetail} = $obj->{acl_ntfydetail};
     $me->{timezone}       = $obj->{notify}{timezone};
     $me->{mailfrom}       = $obj->{notify}{mail_from};
+    $me->{unack_to}       = $obj->{notify}{unack_timeout};
     
     1;
 }
@@ -157,19 +227,24 @@ END {
 sub newid {
     my( $id, $n );
 
-    unless(open(F, "+< $::datadir/notno")){
-	::loggit( "cannot open $::datadir/notno: $!", 1 );
+    if( ::topconf('_test_mode') ){
+	$id = 1;
+	while( exists $byid{$id} ){ $id ++ };
+    }else{
+	if( open(F, "+< $::datadir/notno") ){
+	    chop( $id = <F> );
+	    $id ++;
+	    while( exists $byid{$id} ){ $id = int(rand( 65535 << (++$n/1000) )); }
+	    
+	    seek F, 0, 0;
+	    print F "$id\n";
+	    close F;
+	}else{
+	    ::loggit( "cannot open $::datadir/notno: $!", 1 );
+	    $id = $$;
+	    while( exists $byid{$id} ){ $id = int(rand( 65535 << (++$n/1000) )); }
+	}
     }
-    chop( $id = <F> );
-    $id ++;
-    # make sure not in use
-    while( exists $byid{$id} ){
-	$id = int(rand( 65535 << (++$n/1000) ));
-    }
-    
-    seek F, 0, 0;
-    print F "$id\n";
-    close F;
 
     $id;
 }
@@ -178,6 +253,7 @@ sub save {
     my $me = shift;
     my $f = "$::datadir/notify/$me->{idno}";
 
+    return if ::topconf('_test_mode');
     open( N, "> $f" ) || return $me->loggit( 'system', "cannot open save file: $!", 1 );
 
     print N "idno: $me->{idno}\n";
@@ -248,6 +324,8 @@ sub load {
     $me->init($obj);
 }
 
+################################################################
+
 sub notify {
     my $me  = shift;
     my( $l );
@@ -258,13 +336,15 @@ sub notify {
     
     $me->{sentcnt} ++;
     $me->{lastsent} = $^T;
+    $me->ack() if $me->{autoack};
 }
 
 # I say again! repeated the Pigeon
 #   -- Alice in Wonderland
 sub renotify {
     my $me = shift;
-
+    my( $l );
+    
     # auto-ack if aa and it has already been sent
     return $me->ack() if $me->{autoack};
 
@@ -287,6 +367,9 @@ sub escalate {
     my $me = shift;
     my( $n );
 
+    # auto-ack if aa and it has already been sent
+    return $me->ack() if $me->{autoack};
+    
     # postpone if no longer down for X minutes
     if( $me->{obj}->{status} ne 'down'
 	&& $^T - $me->{obj}->{transtime} > $DELAY_ESCALATE ){
@@ -309,8 +392,9 @@ sub sendorqueue {
     my( $qt );
 
     $qt = NotMe::qtime($dst);
+    $qt = ::topconf('qtime') unless defined $qt;
     $qt = $QUEUETIME unless defined $qt;
-    
+
     if( $lastsent{$dst} && ($lastsent{$dst} + $qt > $^T) ){
 	$me->queue($dst, $tag);
     }else{
@@ -364,6 +448,7 @@ sub ack {
     
     # remove from unacked
     delete $unacked{ $me->{idno} };
+    $lastupd = $^T;
     
     # stats, ...
 
@@ -423,15 +508,27 @@ sub maintenance {
 	    next;
 	}
 	# auto-ack if no longer down and TO
-	if( $p->{obj}->{status} ne 'down' && $^T - $p->{obj}->{transtime} > $UNACK_TO ){
+	if( $p->{obj}->{status} ne 'down' && $p->{unack_to} && $^T - $p->{obj}->{transtime} > $p->{unack_to} ){
 	    $p->ack('timeout');
 	    next;
 	}
 	# auto-ack if not down and ackonup is set
-	if( $p->{obj}->{status} ne 'down' && $p->{ackonup} ){
-	    $p->ack('up');
+	if( $p->{objstate} eq 'down' && $p->{obj}->{status} ne 'down' && $p->{ackonup} ){
+	    $p->ack('ackonup');
+	}
+	# auto-ack on severity changes
+	if( $p->{severity} ne $p->{obj}->{currseverity} ){
+	    my $old = $MonEl::severity_sort{ $p->{severity} };
+	    my $new = $MonEl::severity_sort{ $p->{obj}->{currseverity} };
+
+	    if( $old > $new && $p->{ackonbetter} ){
+		$p->ack('ack_on_better');
+	    }
+	    if( $old < $new && $p->{ackonworse} ){
+		# hopefully, the user has configured a new notification at this severity
+		$p->ack('ack_on_worse');
+	    }
 	}
-	
 	if( $p->{period} && ($p->{lastsent} + $p->{period} <= $^T) ){
 	    # time to resend
 	    $p->renotify();
@@ -449,6 +546,7 @@ sub maintenance {
 	# print STDERR "sendq? $dst\n";
 	# time to send?
 	my $qt = NotMe::qtime($dst);
+	$qt = ::topconf('qtime') unless defined $qt;
 	$qt = $QUEUETIME unless defined $qt;
 	
 	if( $lastsent{$dst} + $qt <= $^T ){
@@ -479,7 +577,7 @@ sub janitor {
 	    my $f = "$::datadir/notify/$n->{idno}";
 	    unlink $f;
 	    $n->{obj}{notify}{list} = [ grep { $_ != $n }
-					@{$obj->{notify}{list}} ];
+					@{$n->{obj}{notify}{list}} ];
 	    $n->{log} = $n->{sendto} = $n->{status} = $n->{obj} = undef
 	}else{
 	    $n_byid{ $n->{idno} } = $n;
@@ -514,27 +612,59 @@ sub transmit {
     my @more = @_;
     my( $msg, $extra );
 
+    # is dst disabled?
+    if( $dstparam{$dst} && $dstparam{$dst}{disabled} ){
+	# $me->loggit( $dst, 'not sent' );
+	$me->{status}{$dst} = 'disabled';
+	
+	return;
+    }
+    
     # if many, summarize
     if( @more ){
-	my $d;
-	foreach my $p (@more){
-	    $d = 1 if $p->{objstate} eq 'down';
-	}
-	if( $d ){
-	    $msg = $Conf::Config{message_lotsdn} || 'Lots of stuff just went DOWN!';
+	
+	my $nolots = NotMe::nolots( $dst );
+	$nolots ||= ::topconf('nolotsmsgs');
+	
+	if( $nolots ){
+	    # list all messages, don't summarize into Lots UP/DOWN
+
+	    my $j;
+	    if( grep { /\n/s } $msg, @more ){
+		$j = "\n\n";
+	    }else{
+		$j = "\n";
+	    }
+	    
+	    $msg = join($j, map { $_->{msg} } $msg, @more);
 	}else{
-	    $msg = $Conf::Config{message_lotsup} || 'Oodles and oodles just came UP!';
+	    my $d;
+	    foreach my $p (@more){
+		$d = 1 if $p->{objstate} eq 'down';
+	    }
+	    if( $d ){
+		$msg = ::topconf('message_lotsdn') || 'Lots of stuff just went DOWN!';
+	    }else{
+		$msg = ::topconf('message_lotsup') || 'Lots of stuff just came UP!';
+	    }
 	}
     }else{
-	$msg = "$me->{idno} $me->{msg}";
+	$msg = $me->{msg};
     }
 
     $extra .= " $tag" if $tag;
     $extra .= " ESCALATED" if $me->{escalated};	# RSN - from $who...
 
-    NotMe::transmit($me, $dst, $msg, $extra)
-	unless $Conf::Config{_dont_ntfy};
+    my $err = ::topconf('_dont_ntfy') ? 0 : 
+      NotMe::transmit($me, $dst, $msg, $extra, @more);
 
+    if( $err ){
+	foreach my $p ( $me, @more ){
+	    $p->loggit( $dst, 'transmit-failed' );
+	}
+	return;
+    }
+    
     $lastsent{$dst} = $^T;
     foreach my $p ( $me, @more ){
 	$p->loggit( $dst, 'transmit' );
@@ -548,26 +678,58 @@ sub transmit {
 }
 
 sub expand {
+    my $me  = shift;
+    my $fmt = shift;
     my $msg = shift;
     my $obj = shift;
 
-    $msg =~ s/%r/$obj->{srvc}{reason}/g;
-    $msg =~ s/%v/$obj->{test}{currvalue}/g;
+    # interpolate message 1st, it may contain additional % sequences...
+    $fmt =~ s/%m/$msg/g;
 
-    $msg;
+    # pretify value
+    my $v = $obj->{srvc} ? $obj->{srvc}{result} : undef;
+    if( length($v) > 40 ){
+	$v = substr($v,0,40);
+	$v .= ' [...]';
+    }
+    $v =~ s/\r/\\r/gs;
+    $v =~ s/\n/\\n/gs;
+    
+    my $res = $obj->expand($fmt,
+	localtime	=> 1,
+	dtformat	=> "%d/%b %R",	# dd/Mon hh:mm
+	time		=> $me->{created},
+	data		=> {
+	    i	=> $me->{idno},
+	    o 	=> $obj->{uname},
+	    p	=> $me->{priority},
+	    s	=> uc($me->{objstate}),
+	    v   => $v,
+	    y	=> $me->{severity},
+	},
+    );
+
+    $res;
 }
 
 ################################################################
 
 sub number_of_notifies { scalar keys %byid }
+sub byid { my $id = shift; $byid{$id} }
+sub unacked { \%unacked }
+sub lastupd { $lastupd }
+
+################################################################
 
 sub cmd_list_line {
     my $me = shift;
     my $ctl = shift;
     
-    $ctl->write("$me->{idno} $me->{objstate} $me->{created} " .
+    $ctl->write("$me->{idno} $me->{state} $me->{objstate} $me->{created} " .
 		encode($me->{obj}->filename()) . " " .
-		encode($me->{msg}) . "\n");
+		encode($me->{msg} || '_') . ' ' .
+		encode($me->{priority} || '_') . ' '. encode($me->{severity} || '_') .
+		"\n");
 }
 
 sub cmd_list {
@@ -593,49 +755,58 @@ sub cmd_list {
     else{
 	# newest first
 	foreach my $p (sort {$b->{created} <=> $a->{created}} values %byid){
-	    $p->cmd_list_line($ctl);
+	    $p->cmd_list_line($ctl)
+		unless $p->{audit} && !$param->{showaudit};
 	}
     }
-    $ctl->write("\n");
+    $ctl->final();
 }
 
 sub cmd_detail {
     my $ctl = shift;
     my $param = shift;
-    my( $p, $k, $v, $ww, $n );
 
-    $p = $param->{idno};
-    if( exists $byid{$p} ){
-	$p = $byid{$p};
-	$ctl->ok();
-	foreach $k (sort keys %$p){
-	    $v = $p->{$k};
-	    if( ref($v) ){
-		$v = "#<REF>";
-	    }else{
-		$v = encode( $v );
-	    }
-	    $ctl->write( "$k:\t$v\n" );
-	}
-	$ctl->write( "object: " . encode( $p->{obj}->unique() ) . "\n" );
-
-	foreach my $w (sort keys %{$p->{status}}){
-	    my $we = encode($w);
-	    $ww .= " $we";
-	    $ctl->write("status $we: " . $p->{status}{$w} . "\n" );
-	}
-	$ctl->write("statuswho: $ww\n");
-	$n = 0;
-	foreach my $l (@{$p->{log}}){
-	    $ctl->write( "log $n: $l->{time} " . ($l->{who}? encode($l->{who}) : '_') .
-			 ' ' . encode($l->{msg}) . "\n" );
-	    $n++;
-	}
-	$ctl->write( "loglines: $n\n");
-	$ctl->write( "\n" );
-    }else{
+    my $idno = $param->{idno};
+    unless( exists $byid{$idno} ){
 	$ctl->bummer(404, 'Notification Not Found');
+	return;
     }
+	
+    my $p = $byid{$idno};
+    $ctl->ok();
+    foreach my $k (sort keys %$p){
+	my $v = $p->{$k};
+	if( ref($v) ){
+	    $v = "#<REF>";
+	}else{
+	    $v = encode( $v );
+	}
+	$ctl->write( "$k:\t$v\n" );
+    }
+    $ctl->write( "object: " . encode( $p->{obj}->unique() ) . "\n" );
+    
+    for my $e (qw(style_sheet javascript bkgimage icon icon_up icon_down)){
+	my $v = $p->{obj}->{web}{$e};
+	next unless $v;
+	$ctl->write("web $e: " . encode($v) . "\n");
+    }
+
+    my $ww;
+    foreach my $w (sort keys %{$p->{status}}){
+	my $we = encode($w);
+	$ww .= " $we";
+	$ctl->write("status $we: " . $p->{status}{$w} . "\n" );
+    }
+    $ctl->write("statuswho: $ww\n");
+    
+    my $n = 0;
+    foreach my $l (@{$p->{log}}){
+	$ctl->write( "log $n: $l->{time} " . ($l->{who}? encode($l->{who}) : '_') .
+		     ' ' . encode($l->{msg}) . "\n" );
+	$n++;
+    }
+    $ctl->write( "loglines: $n\n");
+    $ctl->final();
 }
 
 sub cmd_ack {
@@ -645,6 +816,12 @@ sub cmd_ack {
     
     $p = $param->{idno};
     $u = $param->{user} || 'anonymous';
+
+    if( defined &notify_ack_policy ){
+	unless( notify_ack_policy($param) ){
+	    $ctl->bummer(401, 'Not Permitted');
+	}
+    }
     
     if( $p eq 'all' ){
 	$ctl->ok_n();
@@ -660,14 +837,61 @@ sub cmd_ack {
     }
 }
 
+sub cmd_set_dst {
+    my $ctl = shift;
+    my $param = shift;
+
+    my $dst = $param->{dst};
+    my $p   = $param->{param};
+    my $v   = $param->{value};
+
+    if( defined($dst) && defined($p) ){
+	if( $p eq 'disable' || $p eq 'redirect' ){
+	    if( $v ){
+		$dstparam{$dst}{$p} = $v;
+	    }else{
+		delete $dstparam{$dst}{$p};
+	    }
+	    $ctl->ok_n();
+	}else{
+	    $ctl->bummer(404, 'Unknown param' );
+	}
+    }else{
+	$ctl->bummer(500, 'Missing Param');
+    }
+}
+
+sub cmd_get_dst {
+    my $ctl = shift;
+    my $param = shift;
+
+    my $dst = $param->{dst};
+    my $p   = $param->{param};
+
+    if( defined($dst) && defined($p) ){
+	my $v;
+	if( exists $dstparam{$dst} && exists $dstparam{$dst}{$p} ){
+	    $v = $dstparam{$dst}{$p};
+	}
+	$ctl->ok();
+	$ctl->write( "value: " . encode($v) . "\n\n" );
+    }else{
+	$ctl->bummer(500, 'Missing Param');
+    }
+}
+
 ################################################################
 
-Control::command_install( 'notify_list',     \&cmd_list,
+Control::command_install( 'notify_list',   \&cmd_list,
 			  'list notifications', 'which' );
 Control::command_install( 'notify_detail', \&cmd_detail,
 			  'tell about notification', 'idno' );
-Control::command_install( 'notify_ack', \&cmd_ack,
+Control::command_install( 'notify_ack',    \&cmd_ack,
 			  'ack a notification', 'idno user' );
+Control::command_install( 'notify_setparam', \&cmd_set_dst,
+			  'modify per dst params', 'dst param value' );
+Control::command_install( 'notify_getparam', \&cmd_get_dst,
+			  'get per dst param', 'dst param' );
 
 Cron->new( freq => 60,
 	   text => 'Notification maintenance',
