@@ -5,9 +5,11 @@
 # Date: 2002-Apr-06 21:30 (EST)
 # Function: web page stuff
 #
-# $Id: Web.pm,v 1.40 2003/04/04 19:28:48 jaw Exp $
+# $Id: Web.pm,v 1.71 2008/09/14 15:37:55 jaw Exp $
 
 package MonEl;
+use strict;
+use vars qw(%byname);
 
 # they could hear Circe within, singing most beautifully as she worked
 # at her loom, making a web so fine, so soft, and of such dazzling colours
@@ -17,10 +19,11 @@ package MonEl;
 sub web_build {
     my $me = shift;
     my $topp = shift;
-    my( $h, $file, $fh, $l );
+    my( $h, $file, $fh );
 
+    return undef if ::topconf('_test_mode');
     $h = $topp ? 'bldtimetop' : 'bldtime';
-    $file = "$::datadir/html/" . $me->filename() . ($topp ? '.top' : '.base');
+    $file = "$::datadir/html/" . $me->pathname() . ($topp ? '.top' : '.base');
     return $file if $me->{web}{$h} > $me->{web}{transtime};
     
     $topp = 0 if $me->{web}{alwaysbase};
@@ -49,61 +52,101 @@ sub web_build {
 	
 }
 
-sub web_top {
-    my $me = shift;
-    my $fh = shift;
-    my $topp = shift;
-    my( $icon, $n, $wav, $t, $color );
+sub web_header {
+    my $me    = shift;
+    my $fh    = shift;
+    my $title = shift;
+    my $id    = shift;
+    
+    $title ||= $me->unique();
+    $id    ||= $me->filename();
     
-    $n = $me->unique();
-
     my $refresh = $me->{web}{refresh};
     my $bkgimg  = $me->{web}{bkgimage};
     $bkgimg = "BACKGROUND=\"$bkgimg\"" if $bkgimg;
 
     print $fh "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n";
-    print $fh "<!-- start of web_top -->\n";
-    print $fh "<HTML><HEAD><TITLE>Argus - $n</TITLE>\n";
+    print $fh "<!-- start of web_header -->\n";
+    print $fh "<!-- cachestale: $me->{web}{cachestale} -->\n";
+    print $fh "<HTML><HEAD><TITLE>Argus - $title</TITLE>\n";
     print $fh "<META HTTP-EQUIV=\"REFRESH\" CONTENT=\"$refresh\">\n" if $refresh;
+
+    my $icon = $me->{web}{"icon.$me->{currseverity}"};
     if( $me->{ovstatus} eq 'up' ){
-	$icon = $me->{web}{icon_up};
+	$icon ||= $me->{web}{icon_up};
     }elsif( $me->{ovstatus} eq 'down' ){
-	$icon = $me->{web}{icon_down};
+	$icon ||= $me->{web}{icon_down};
     }
     $icon ||= $me->{web}{icon};
     print $fh "<LINK REL=\"icon\" HREF=\"$icon\" TYPE=\"image/gif\">\n"
 	if $icon;
+    
+    # permit mutiple style sheets + javascripts
+    for my $ss ( split /\s+/, $me->{web}{style_sheet} ){
+	print $fh "<LINK REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"$ss\">\n";
+    }
+    for my $js ( split /\s+/, $me->{web}{javascript} ){
+	print $fh "<SCRIPT TYPE=\"text/javascript\" SRC=\"$js\"></SCRIPT>\n";
+    }
+    print $fh "</HEAD><BODY $bkgimg BGCOLOR=\"#FFFFFF\" ID=\"$id\">\n";
 
-    print $fh "</HEAD><BODY $bkgimg BGCOLOR=\"#FFFFFF\">\n";
-    
-    print $fh "<!-- START AUTHORIZATION READ NEEDED $me->{acl_page} -->\n";
-    
+    print $fh "<DIV CLASS=HEADER>\n";
     print $fh "$me->{web}{header_all}\n" if $me->{web}{header_all};
     print $fh "$me->{web}{header}\n" if $me->{web}{header};
+    print $fh "</DIV>\n";
+    print $fh "<!-- end of web_header -->\n";
+    
+}
+    
+sub web_show_has_errors {
+    my $me = shift;
+    my $fh = shift;
+    my $nc = shift;
+    
+    print $fh <<X;
+        <TR BGCOLOR="#FF8888"><TD COLSPAN=$nc>
+	<TABLE CLASS=WARNING><TR><TD>
+	<H2>Warning&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</H2>
+	</TD><TD>
+	<B>Errors detected during startup</B><BR>
+	Attempting to run anyway<BR>
+	Examine the <A HREF="__BASEURL__?func=logfile;abridge=1">Error Log</A> for details
+	</TD></TR></TABLE>
+	</TD></TR>
+X
+    ;
+}
+
+sub web_top {
+    my $me = shift;
+    my $fh = shift;
+    my $topp = shift;
+    my( $wav, $t, $color );
+    
+    my $n = $me->unique();
+    my $file = $me->filename();
+
+    print $fh "<!-- start of web_top -->\n";
+    $me->web_header($fh);
+    print $fh "<!-- START AUTHORIZATION READ NEEDED $me->{acl_page} -->\n";
 
     if( $me->{alarm} ){
 	$t = $me->{sirentime};
 	$wav = $me->{web}{sirensong};
 	if( $wav ){
+	    # Song that as greatly doth transcend our Muses,
+	    # Our Sirens, in those dulcet clarions,
+	    # As primal splendour that which is reflected.
+	    #   -- Dante, Divine Comedy
+	    
 	    # cgi will do browser detection and re-write this
 	    print $fh "<!-- START SIREN $t $wav -->\n";
 	}
     }
 
-    print $fh "<TABLE WIDTH=\"95%\" BORDER=1>\n";
+    print $fh "<TABLE WIDTH=\"95%\" BORDER=1 CLASS=MAIN>\n";
     if( $Conf::has_errors && $topp ){
-	print $fh <<X;
-        <TR BGCOLOR="#FF8888"><TD COLSPAN=2>
-	<TABLE><TR><TD>
-	<H2>Warning&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</H2>
-	</TD><TD>
-	<B>Errors detected during startup</B><BR>
-	Attempting to run anyway<BR>
-	Examine the <A HREF="__BASEURL__?func=logfile">Error Log</A> for details
-	</TD></TR></TABLE>
-	</TD></TR>
-X
-    ;
+	$me->web_show_has_errors($fh, 2);
     }
     
     if( $Conf::has_errors ){
@@ -116,8 +159,8 @@ X
     $nospkr = "<IMG SRC=\"$nospkr\" ALT=\"speaker off\">" if $nospkr;
     print $fh <<X;
 <TR BGCOLOR="#$color"><TD COLSPAN=2>
-  <TABLE BORDER=0 WIDTH="100%">
-    <TR> <TD ALIGN=LEFT>$n</TD> <TD ALIGN=RIGHT>User: <TT>__USER__</TT>
+  <TABLE BORDER=0 WIDTH="100%" CLASS=TOPBAR>
+    <TR> <TD ALIGN=LEFT>$n</TD> <TD ALIGN=RIGHT><L10N User>: <TT>__USER__</TT>
 	<!-- SIREN ICON -->$nospkr
 	</TD> </TR>
   </TABLE>
@@ -141,12 +184,28 @@ sub web_bottom {
     $me->web_side_buttons($fh, $topp);
     print $fh "</TD></TR>\n</TABLE>\n";
 
+    print $fh "<!-- END AUTHORIZATION READ NEEDED -->\n";
+
+    $me->web_arjax($fh, $topp) if $me->{web}{javascript};
+    
+    $me->web_footer($fh);
+    print $fh "<!-- end of web_bottom -->\n";
+}
+
+sub web_footer {
+    my $me = shift;
+    my $fh = shift;
+
+    print $fh "<!-- start of web_footer -->\n";
+    print $fh "<DIV CLASS=FOOTER>\n";
     print $fh "$me->{web}{footer}\n" if $me->{web}{footer};
     print $fh "$me->{web}{footer_all}\n" if $me->{web}{footer_all};
-    print $fh "<!-- END AUTHORIZATION READ NEEDED -->\n";
-    print $fh "$me->{web}{footer_argus}\n" if $me->{web}{footer_argus};
+    print $fh "</DIV>\n";
+    print $fh "<DIV CLASS=FOOTERARGUS>$me->{web}{footer_argus}</DIV>\n"
+	if $me->{web}{footer_argus};
+
     print $fh "</BODY>\n";
-    print $fh "<!-- end of web_bottom -->\n";
+    print $fh "<!-- end of web_footer -->\n";
     print $fh "</HTML>\n";
 }
 
@@ -156,15 +215,17 @@ sub web_logs {
     my $me = shift;
     my $fh = shift;
 
-    return undef unless @{$me->{stats}{log}};
+    return undef unless $me->{stats}{log} && @{$me->{stats}{log}};
     print $fh "<!-- start of web_logs -->\n";
-    print $fh "<TABLE>\n";
+    print $fh "<B>", $me->unique(), " <L10N state transitions></B><BR>\n";
+    print $fh "<TABLE CLASS=LOGS>\n";
+    print $fh "<tr><th><L10N when></th><th><L10N state></th><th><L10N reason></th></tr>\n";
+    
     foreach my $l ( reverse @{$me->{stats}{log}} ){
 	# [time, status, ovstatus, tag, msg]
-	my $st = $l->[1];
-	$st .= "/$l->[2]" if $l->[1] ne $l->[2];
+	my $st = $l->[2];
 	print $fh "<TR><TD ALIGN=RIGHT><LOCALTIME $l->[0]> ", 
-	    "</TD><TD> $st </TD><TD> $l->[3] - $l->[4]</TD></TR>\n";
+	    "</TD><TD> <L10N $st> </TD><TD> $l->[3] - $l->[4]</TD></TR>\n";
 
     }
     print $fh "</TABLE>\n";
@@ -215,9 +276,11 @@ sub web_stat_line {
     $e = $x->{elapsed};
     $color = $n ? '' : ' BGCOLOR="#AAAAFF"';
     
-    print $fh "<TR$color><TD>$label </TD><TD ALIGN=RIGHT>&nbsp; <LOCALTIME $x->{start}> </TD><TD ALIGN=RIGHT> ",
+    print $fh "<TR$color><TD><L10N $label> </TD><TD ALIGN=RIGHT>&nbsp; <LOCALTIME $x->{start}>",
+    	"</TD><TD ALIGN=RIGHT> ",
     elapsed($x->{elapsed}), " </TD><TD ALIGN=RIGHT>&nbsp; ", percent($x->{up}, $e),
-    "</TD><TD ALIGN=RIGHT>&nbsp; ", percent($x->{down}, $e), "</TD><TD ALIGN=RIGHT> $x->{ndown}</TD></TR>\n";
+    "</TD><TD ALIGN=RIGHT>&nbsp; ", percent($x->{down}, $e),
+    "</TD><TD ALIGN=RIGHT> $x->{ndown}</TD></TR>\n";
 
 }
 
@@ -227,9 +290,10 @@ sub web_stats {
     
     print $fh "<!-- start of web_stats -->\n";
 
-    print $fh "<B>Status: $me->{status} since <LOCALTIME $me->{transtime}></B><BR>\n";
-    print $fh "<TABLE BORDER=0 CELLSPACING=0>\n";
-    print $fh "<TR><TH>&nbsp;</TH><TH>start</TH><TH>elapsed time</TH><TH>% up</TH><TH>% down</TH><TH>times down</TH></TR>\n";
+    print $fh "<B><L10N Status>: <L10N $me->{status} since> <LOCALTIME $me->{transtime}></B><BR>\n";
+    print $fh "<TABLE BORDER=0 CELLSPACING=0 CLASS=STATS>\n";
+    print $fh "<TR><TH>&nbsp;</TH><TH><L10N start></TH><TH><L10N elapsed time></TH>",
+    "<TH>% <L10N up></TH><TH>% <L10N down></TH><TH><L10N times down></TH></TR>\n";
     
     $me->web_stat_line($fh, 'Today', 'daily', 0);
     $me->web_stat_line($fh, 'Yesterday', 'daily', 1);
@@ -254,15 +318,15 @@ sub web_override {
     
     return unless $me->{override};
     print $fh "<!-- start of web_override -->\n";
-    print $fh "<HR>\n<B>Override</B>\n<TABLE>\n";
+    print $fh "<HR>\n<B><L10N Override></B>\n<TABLE CLASS=OVERRIDE>\n";
     foreach my $k (qw(user time expires mode ticket text)){
 	my $v = $me->{override}{$k};
 	next unless $v;
 	$v = "<LOCALTIME $v>" if( $k eq 'time' || $k eq 'expires' );
-	if( $k eq 'ticket' && $me->can('tkt_web_value') ){
-	    $v = $me->tkt_web_value($v);
+	if( $k eq 'ticket' && $me->can('tkt_watch_web') ){
+	    $v = $me->tkt_watch_web($v);
 	}
-	print $fh "<TR><TD>&nbsp;</TD><TD>$k</TD><TD>$v</TD></TR>\n";
+	print $fh "<TR><TD>&nbsp;</TD><TD><L10N $k></TD><TD>$v</TD></TR>\n";
     }
 	
     
@@ -270,13 +334,21 @@ sub web_override {
     print $fh "<!-- end of web_override -->\n";
 }
 
+
 sub web_button {
     my $me    = shift;
     my $label = shift;
     my $color = shift || 'DDDDDD';
     my $link  = $me->url(@_);
 
-    web_button_text( "<A HREF=\"$link\">$label</A>", $color )
+    my $id;
+    foreach my $p (@_){
+	my($func) = $p =~ /func=(.*)/;
+	$id ||= $func;
+    }
+
+    $id = qq( id="button_$id") if $id;
+    web_button_text( "<A$id HREF=\"$link\"><L10N $label></A>", $color )
 }
 
 sub web_button_no {
@@ -286,14 +358,15 @@ sub web_button_no {
     my $link  = join( ';', @_ );
 
     $link = '__BASEURL__' . ($link ? '?' : '') . $link;
-    web_button_text( "<A HREF=\"$link\">$label</A>", $color )
+    web_button_text( "<A HREF=\"$link\"><L10N $label></A>", $color )
 }
 
 sub web_button_text {
     my $txt   = shift;
     my $color = shift || 'DDDDDD';
-    
-    "<TABLE BORDER=1 WIDTH=\"100%\"><TR><TD ALIGN=CENTER BGCOLOR=\"#$color\">".
+
+    my $class = ($color eq 'FF8888') ? 'ERRORSBUTTON' : 'BUTTON';
+    "<TABLE BORDER=1 WIDTH=\"100%\" CLASS=BUTTON><TR><TD CLASS=$class ALIGN=CENTER BGCOLOR=\"#$color\">".
     "$txt</TD></TR></TABLE>\n";
 }
 
@@ -302,8 +375,9 @@ sub web_side_buttons {
     my $fh = shift;
     my $topp = shift || 0;
 
+    my $top = $byname{Top};
     print $fh $me->{web}{buttons_top_html}, "\n" if $me->{web}{buttons_top_html};
-    
+
     unless( $topp ){
 	if( $me->{overridable} ){
 	    print $fh "<!-- START AUTHORIZATION RW NEEDED $me->{acl_override} -->\n";
@@ -317,16 +391,17 @@ sub web_side_buttons {
 	
 	print $fh "<!-- START AUTHORIZATION RW NEEDED $me->{acl_annotate} -->\n";
 	if( $me->{annotation} ){
-	    print $fh $me->web_button('Remove Annotation', undef, 'func=rmannotate');
+	    print $fh $me->web_button('Remove Annotation', undef, 'func=rmannotate', 'phase=1');
 	}else{
 	    print $fh $me->web_button('Annotate', undef, 'func=annotate');
 	}
 	print $fh "<!-- END AUTHORIZATION RW NEEDED -->\n";
 	
-	print $fh "<!-- START AUTHORIZATION RW NEEDED $me->{acl_flush} -->\n";
-	print $fh $me->web_button('Flush Cache', undef, 'func=flushcache');
+	print $fh "<!-- START AUTHORIZATION RW NEEDED $me->{acl_checknow} -->\n";
+	print $fh $me->web_button('Check Now', undef, 'func=checknow');
 	print $fh "<!-- END AUTHORIZATION RW NEEDED -->\n";
     }
+    
     print $fh "<!-- START AUTHORIZATION RW NEEDED $me->{acl_getconf} -->\n";
     print $fh $me->web_button('Display Config', undef, 'func=dispconf');
     print $fh "<!-- END AUTHORIZATION RW NEEDED -->\n";
@@ -336,20 +411,22 @@ sub web_side_buttons {
     print $fh "<!-- END AUTHORIZATION DEBUG NEEDED -->\n";
 
     # paging...
-    print $fh "<!-- START AUTHORIZATION RW NEEDED $me->{acl_ntfylist} -->\n";
+    print $fh "<!-- START AUTHORIZATION RW NEEDED $top->{acl_ntfylist} -->\n";
     print $fh $me->web_button_no('Un-Acked Notifies', undef, 'func=ntfylsua');
     print $fh $me->web_button_no('Notifies', undef, 'func=ntfylist');
     print $fh "<!-- END AUTHORIZATION RW NEEDED -->\n";
 
+    print $fh $me->web_button_no('Overview', undef, 'object=overview', 'func=page');
+
     # error log
-    print $fh "<!-- START AUTHORIZATION RW NEEDED $me->{acl_logfile} -->\n";
+    print $fh "<!-- START AUTHORIZATION RW NEEDED $top->{acl_logfile} -->\n";
     print $fh $me->web_button_no('Error Log',
 				 $Conf::has_errors ? 'FF8888' : undef,
-				 'func=logfile',
+				 'func=logfile', 'abridge=1',
 				 );
     print $fh "<!-- END AUTHORIZATION RW NEEDED -->\n";
 
-    if( $me->{alarm} && $Conf::Config{sirensong} ){
+    if( $me->{alarm} && $me->{web}{sirensong} ){
 	my $t = $me->{sirentime};
 	print $fh "<!-- START SIRENBUTTON $t -->\n";
 	if( $topp ){
@@ -375,13 +452,6 @@ sub web_side_buttons {
     }
 
     print $fh "<BR>\n";
-
-# RSN - web based configurator
-#    print $fh "<!-- START AUTHORIZATION CONF NEEDED $me->{acl_conf} -->\n";
-#    print $fh $me->web_button('Add Data',    undef, 'func=adddata');
-#    print $fh $me->web_button('Add Group',   undef, 'func=addgrp')  if $me->{type} =~ /Group|Host/;
-#    print $fh $me->web_button('Add Service', undef, 'func=addsrvc') if $me->{type} =~ /Group|Host/;
-#    print $fh "<!-- END AUTHORIZATION CONF NEEDED -->\n";
     
     print $fh $me->{web}{buttons_bottom_html}, "\n" if $me->{web}{buttons_bottom_html};
 
@@ -395,41 +465,18 @@ sub web_page_row_top {
     my $me = shift;
     my $fh = shift;
     my $label = shift;
-    my( %cs );
 
     return if $me->{web}{hidden};
     return unless @{$me->{children}};
     print $fh "<TR><TD><A HREF=\"", $me->url('func=page'), "\">",
         ($label||$me->{label_left}||$me->{name}), "</A></TD>";
 
-    for my $c (@{$me->{children}}){
-	my $cc = $c;
-	$cc = $c->aliaslookup() if $c->can('aliaslookup');
-
-	if( $cc->{countstop} ){
-	    $cs{ $cc->{ovstatus} } ++;
-	    next;
-	}
-
-	if( ! @{$cc->{children}} ){
-	    $cs{ $cc->{ovstatus} } ++;
-	    next;
-	}
-	
-	if( $cc->{override} ){
-	    $cs{override} += @{$cc->{children}};
-	    next;
-	}
-
-	foreach my $gc ( @{$cc->{children}} ){
-	    my $gcc = $gc;
-	    $gcc = $gc->aliaslookup() if $gc->can('aliaslookup');
-	    $cs{ $gcc->{ovstatus} } ++;
-	}
-    }
+    my $smy = $me->summary();
+    my $csv = $smy->{severity};
+    
     foreach my $s ('up', 'down', 'override'){
-	my $color = $cs{$s} ? (" BGCOLOR=\"" . MonEl::web_color($s) . "\"" ) : '';
-	print $fh "<TD WIDTH=\"25%\" ALIGN=RIGHT$color>", ($cs{$s}||0), "</TD>";
+	my $color = $smy->{$s} ? (" BGCOLOR=\"" . MonEl::web_color($s, $csv) . "\"" ) : '';
+	print $fh "<TD WIDTH=\"25%\" ALIGN=RIGHT$color>", ($smy->{$s}||0), "</TD>";
     }
     print $fh "</TR>\n";
 }
@@ -438,14 +485,14 @@ sub web_notifs {
     my $me = shift;
     my $fh = shift;
 
-    return unless @{$me->{notify}{list}};
+    return unless $me->{notify}{list} && @{$me->{notify}{list}};
     return unless $me->{web}{shownotiflist};
     print $fh "<!-- start of web_notifs -->\n";
     print $fh "<!-- START AUTHORIZATION RW NEEDED $me->{acl_ntfydetail} -->\n";
-    print $fh "<B>Notifications:</B> \n";
+    print $fh "<B><L10N Notifications>:</B> \n";
     foreach my $n ( @{$me->{notify}{list}} ){
 	my $link = "__BASEURL__?func=ntfydetail;idno=$n->{idno}";
-	my $color = web_color( $n->{objstate} );
+	my $color = web_color( $n->{objstate}, $n->{severity} );
 	print $fh "<A HREF=\"$link\"><FONT COLOR=$color>$n->{idno}</FONT></A>\n";
     }
     print $fh "<BR>\n<HR>\n";
@@ -457,21 +504,20 @@ sub web_notifs {
 sub web_graphs {
     my $me = shift;
     my $fh = shift;
-    my( $n );
     
-    return unless $::HAVE_GD;
     return unless $me->{graph};
-    return if $Conf::Config{_no_images};
+    return if ::topconf('_no_images');
 
     print $fh "<!-- start of web_graphs -->\n";
-    print $fh "<CENTER><TABLE BORDER=0><TR>\n";
+    print $fh "<CENTER><TABLE BORDER=0 CLASS=GRAPHS><TR>\n";
+    my @p;
+    push @p, ht => $me->{image}{gr_height} if $me->{image}{gr_height};
     for my $w (qw(samples hours days)){
-	$n++;
-	next unless $me->{image}{whichgraphs} =~ /$n/;
-	my $url = $me->url('func=graph', "which=$w", 'size=thumb', 'ext=.png');
+	next unless $me->{image}{"gr_show_$w"};
+	my $url = $me->url('func=graph', "which=$w", 'size=thumb', @p, 'ext=.png');
 	my $big = $me->url('func=graphpage', "which=$w", 'size=full' );
 	print $fh "\t<TD><A HREF=\"$big\"><IMG HEIGHT=64 WIDTH=160 BORDER=0 ",
-	"SRC=\"$url\" ALT=\"graph\"></A><BR><CENTER>$w</CENTER></TD>\n";
+	"SRC=\"$url\" ALT=\"graph\"></A><BR><CENTER><A HREF=\"$big\"><L10N $w></A></CENTER></TD>\n";
     }
     print $fh "</TR></TABLE></CENTER>\n";
     print $fh "<HR>\n";
@@ -484,26 +530,115 @@ sub web_graphs {
 # And drew a thousand colors from the light; 
 #   -- Virgil, Aeneid
 sub web_color {
-    my $s = shift;
+    my $st = shift;
+    my $sv = shift() || 'critical';
 
-    { up       => '#24C926', # green
-      down     => '#FF0012', # red
-      override => '#DDDDDD', # gray
-      depends  => '#FFA500', # orange,  or '#EEEE00', yellow(ish)
-  }->{$s};
+    if( $st eq 'down' ){
+	{ clear    => '#24C926', # green
+	  warning  => '#2424FF', # blue
+	  minor    => '#EEEE00', # yellow
+	  major    => '#FFA500', # orange
+	  critical => '#FF0012', # red
+      }->{$sv};
+    }else{
+	{ up       => '#24C926', # green
+	  down     => '#FF0012', # red
+	  override => '#DDDDDD', # gray
+	  depends  => '#FFA500', # orange,  or '#EEEE00', yellow(ish)
+      }->{$st};
+    }
 }
       
+sub web_arjax {
+    my $me = shift;
+    my $fh = shift;
+    my $topp = shift;
+
+    my $obj = $me->filename();
+
+    print $fh <<EOFORM;
+    <!-- START AUTHORIZATION RW NEEDED $me->{acl_annotate} -->
+    <div id="annotatediv" style="display: none;"><div class=inner>
+    <form name=annotateform method=post action="__BASEURL__">
+    <input type=hidden name=func value=annotate>
+    <input type=hidden name=phase value=1>
+    <input type=hidden name=object value="$obj">
+    <center><b><L10N Annotation></b></center><center><input type=text name=text size=32></center>
+    <br><center><input type=submit> <input id=annotatecancel type=button value=Cancel></center>
+    </form>
+    </div></div>
+    <!-- END AUTHORIZATION RW NEEDED -->
+    
+    <!-- START AUTHORIZATION RW NEEDED $me->{acl_override} -->
+    <div id="overridediv" style="display: none;"><div class=inner>
+    <form name=overrideform method=post action="__BASEURL__">
+    <input type=hidden name=func value=override>
+    <input type=hidden name=phase value=1>
+    <input type=hidden name=object value="$obj">
+
+    <center><b><L10N Override></b></center>
+    <table cellpadding=0 cellspacing=0>    
+    <tr><td><L10N Comment>: </td><td><input type="text" name="text" size="32" /></td></tr>
+EOFORM
+    ;
+
+    # include ticket link if tkt is installed
+    print $fh qq{<tr><td><L10N Ticket No.>: </td><td><input type="text" name="ticket" size="16" /></td></tr>\n}
+        if defined &tkt_watch_add;
+
+    # default to manual if up, else auto
+    my $autop = $me->{ovstatus} eq 'up' ? '' : 'selected=1';
+    my $manup = $autop ? '' : 'selected=1';
+    
+    print $fh <<OVF2;
+<tr><td><L10N Mode>: </td><td><select name="mode">
+<option $autop value="auto"><L10N auto></option>
+<option $manup value="manual"><L10N manual></option></select></td></tr>
+<tr><td><L10N Expires>: </td><td><select name="expires" tabindex="4">
+<option value="0">never</option><option value="900">15 min</option><option value="1800">30 min</option>
+<option value="3600">1 hour</option><option value="7200">2 hours</option><option value="10800">3 hours</option>
+<option selected="selected" value="14400">4 hours</option>
+<option value="21600">6 hours</option><option value="28800">8 hours</option><option value="43200">12 hours</option>
+<option value="64800">18 hours</option><option value="86400">24 hours</option><option value="129600">36 hours</option>
+<option value="172800">2 days</option><option value="259200">3 days</option><option value="345600">4 days</option>
+<option value="432000">5 days</option><option value="604800">7 days</option><option value="864000">10 days</option>
+<option value="1209600">14 days</option><option value="1728000">20 days</option>
+<option value="2592000">30 days</option><option value="3888000">45 days</option>
+</select></td></tr>
+OVF2
+    ;
+
+    print $fh <<EOF2;
+    <tr><td colspan=2><br><center><input type=submit>
+	<input id=overridecancel type=button value=Cancel></center></td></tr>
+    </table>
+    <div id=overridefoot><L10N auto mode - disengage override when status returns to up><br>
+    <L10N manual mode - require override be disengaged manually></div>
+    </form>
+    </div></div>
+    
+    <!-- END AUTHORIZATION RW NEEDED -->
+EOF2
+    ;
+}
 
 ################################################################
 
 sub cmd_webpage {
     my $ctl = shift;
     my $param = shift;
-    my( $x, $f );
+
+    if( $param->{object} eq 'overview' ){
+	my  $f = web_overview();
+	$ctl->ok();
+	$ctl->write( "file: $f\n" );
+	$ctl->write("\n");
+	return;
+    }
     
-    $x = $byname{ $param->{object} };
+    my $x = $MonEl::byname{ $param->{object} };
     if( $x ){
-	$f = $x->web_build( ($param->{top} eq 'yes')? 1 : 0 );
+	my $f = $x->web_build( ($param->{top} eq 'yes')? 1 : 0 );
 	$ctl->ok();
 	$ctl->write( "file: $f\n" );
 	$ctl->write("\n");
@@ -522,7 +657,7 @@ sub cmd_flushpage {
     my $param = shift;
     my( $x );
 
-    $x = $byname{ $param->{object} };
+    $x = $MonEl::byname{ $param->{object} };
     if( $x ){
 	$x->{web}{transtime} = $^T;
 	$ctl->ok_n();
@@ -535,13 +670,15 @@ sub cmd_flushpage {
 sub cmd_logindata {
     my $ctl = shift;
     
-    my $x = $byname{Top};
+    my $x = $MonEl::byname{Top};
     $ctl->ok();
     $ctl->write( "header: ". encode("$x->{web}{header_all}") . "\n");
     $ctl->write( "footer: ". encode("$x->{web}{footer_all} " .
 					  "$x->{web}{footer_argus}") . "\n");
     $ctl->write( "bkgimg: ". encode($x->{web}{bkgimage}) . "\n");
-    $ctl->write("\n");
+    $ctl->write( "style_sheet: ". encode($x->{web}{style_sheet}) . "\n");
+    $ctl->write( "icon: ". encode($x->{web}{icon}) . "\n");
+    $ctl->final();
 }
 
 
